#include "widget.h"
#include "ui_widget.h"

#include <QGridLayout>
#include <QTreeWidget>

#include <QNetworkAccessManager>
#include <QNetworkReply>
#include <QNetworkRequest>
#include <QTimer>

Widget::Widget(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::Widget)
{
    ui->setupUi(this);
    QGridLayout *lo = new QGridLayout(this);
    tw = new QTreeWidget(this);
    tw->setColumnCount(3);
    tw->setHeaderLabels(QStringList() << "method" << "ptr" << "msg");
    lo->addWidget(tw);

    nam = new QNetworkAccessManager(this);

    QNetworkRequest req(QUrl("http://woody.elettra.eu:8001/x/read/ken:20000/seq/test/launch_b/SequencerConfDevices"));
    QNetworkReply *rep1 = nam->get(req);
    char repaddr[128];
    sprintf(repaddr, "%p", rep1);
    new QTreeWidgetItem(tw, QStringList() << "Widget" << repaddr << "requesting " + req.url().toString());
    rep1->setParent(this);
    rep1->setObjectName("rep1");
    connect(rep1, SIGNAL(finished()), this, SLOT(onReplyFinished()));
    connect(rep1, SIGNAL(destroyed(QObject *)), this, SLOT(onReplyDestroyed(QObject *)));
    connect(rep1, SIGNAL(readyRead()), this, SLOT(onNewData()));
    //
    // (1) without this, you can see both replies are correctly delivered
    QTimer::singleShot(200, this, SLOT(onTimeout()));
}

Widget::~Widget()
{
    delete ui;
}

void Widget::onReplyFinished()
{
    QNetworkReply *re = qobject_cast<QNetworkReply *>(sender());
    char repaddr[128];
    sprintf(repaddr, "%p", re);
    new QTreeWidgetItem(tw, QStringList() << "onReplyFinished" << repaddr << "FINISHED WITH ERROR: " + re->errorString());
    re->deleteLater();
}

void Widget::onReplyDestroyed(QObject *re) {
    char repaddr[128];
    sprintf(repaddr, "%p", re);
    new QTreeWidgetItem(tw, QStringList() << "onReplyDestroyed" << repaddr << "DESTROYED");
    if(re->objectName() == "rep2")
        return;
    QNetworkRequest req(QUrl("http://woody.elettra.eu:8001/x/read/ken:20000/seq/test/sequencer_b/State"));

    // (2) this seems to solve... but...
//    nam->clearConnectionCache();
    QNetworkReply *rep2 = nam->get(req);
    sprintf(repaddr, "%p", rep2);
    new QTreeWidgetItem(tw, QStringList() << "Widget" << repaddr << "requesting " + req.url().toString());
    connect(rep2, SIGNAL(finished()), this, SLOT(onReplyFinished()));
    connect(rep2, SIGNAL(destroyed(QObject *)), this, SLOT(onReplyDestroyed(QObject *)));
    connect(rep2, SIGNAL(readyRead()), this, SLOT(onNewData()));
    rep2->setParent(this);
    rep2->setObjectName("rep2");
}

void Widget::onTimeout() {
    QNetworkReply *re = findChild<QNetworkReply*> ("rep1");
    char repaddr[128];
    sprintf(repaddr, "%p", re);
    new QTreeWidgetItem(tw, QStringList() << "onTimeout" << repaddr << "ABORTING......");
    re->abort();
}

void Widget::onNewData()
{
    QNetworkReply *re = qobject_cast<QNetworkReply *>(sender());
    char repaddr[128];
    sprintf(repaddr, "%p", re);
    QByteArray ba = re->readAll();
    new QTreeWidgetItem(tw, QStringList() << "onReplyFinished" << repaddr << "DATA: " + QString(ba));
}

